@@ -16,6 +16,11 @@
 #include "totem-pl-parser-mini.h"
 #include "totem-pl-parser-private.h"
 
+typedef struct {
+	const char *field;
+	char *ret;
+} ParserResult;
+
 static GMainLoop *loop = NULL;
 static gboolean option_no_recurse = FALSE;
 static gboolean option_debug = FALSE;
@@ -246,32 +251,10 @@ static void
 entry_parsed_cb (TotemPlParser *parser,
 		 const char *uri,
 		 GHashTable *metadata,
-		 char **ret)
+		 ParserResult *res)
 {
-	if (*ret == NULL)
-		*ret = g_strdup (uri);
-}
-
-static char *
-parser_test_get_parse_result (const char *uri)
-{
-	TotemPlParserResult retval;
-	char *ret = NULL;
-	TotemPlParser *pl = totem_pl_parser_new ();
-
-	g_object_set (pl, "recurse", !option_no_recurse,
-			  "debug", option_debug,
-			  "force", option_force,
-			  "disable-unsafe", option_disable_unsafe,
-			  NULL);
-	g_signal_connect (G_OBJECT (pl), "entry-parsed",
-			  G_CALLBACK (entry_parsed_cb), &ret);
-
-	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
-	g_test_message ("Got retval %d for uri '%s'", retval, uri);
-	g_object_unref (pl);
-
-	return ret;
+	if (res->ret == NULL)
+		res->ret = g_strdup (g_hash_table_lookup (metadata, res->field));
 }
 
 static void
@@ -368,36 +351,28 @@ simple_parser_test (const char *uri)
 	return retval;
 }
 
-static void
-entry_parsed_genre_cb (TotemPlParser *parser,
-		 const char *uri,
-		 GHashTable *metadata,
-		 char **ret)
-{
-	if (*ret == NULL)
-		*ret = g_strdup (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_GENRE));
-}
-
 static char *
-parser_test_get_parse_genre (const char *uri)
+parser_test_get_entry_field (const char *uri, const char *field)
 {
 	TotemPlParserResult retval;
-	char *ret = NULL;
 	TotemPlParser *pl = totem_pl_parser_new ();
+	ParserResult res;
 
 	g_object_set (pl, "recurse", !option_no_recurse,
 			  "debug", option_debug,
 			  "force", option_force,
 			  "disable-unsafe", option_disable_unsafe,
 			  NULL);
+	res.field = field;
+	res.ret = NULL;
 	g_signal_connect (G_OBJECT (pl), "entry-parsed",
-			  G_CALLBACK (entry_parsed_genre_cb), &ret);
+			  G_CALLBACK (entry_parsed_cb), &res);
 
 	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
 	g_test_message ("Got retval %d for uri '%s'", retval, uri);
 	g_object_unref (pl);
 
-	return ret;
+	return res.ret;
 }
 
 static void
@@ -410,7 +385,7 @@ playlist_started_title_cb (TotemPlParser *parser,
 }
 
 static char *
-parser_test_get_playlist_title (const char *uri)
+parser_test_get_playlist_uri (const char *uri)
 {
 	TotemPlParserResult retval;
 	char *ret = NULL;
@@ -434,71 +409,8 @@ parser_test_get_playlist_title (const char *uri)
 static void
 test_itms_parsing (void)
 {
-	g_assert_cmpstr (parser_test_get_playlist_title ("itms://itunes.apple.com/gb/podcast/best-of-chris-moyles-enhanced/id142102961?ign-mpt=uo%3D4"), ==, "http://downloads.bbc.co.uk/podcasts/radio1/moylesen/rss.xml");
-}
-
-static void
-entry_parsed_id_cb (TotemPlParser *parser,
-		 const char *uri,
-		 GHashTable *metadata,
-		 char **ret)
-{
-	if (*ret == NULL)
-		*ret = g_strdup (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_ID));
-}
-
-static char *
-parser_test_get_id (const char *uri)
-{
-	TotemPlParserResult retval;
-	char *ret = NULL;
-	TotemPlParser *pl = totem_pl_parser_new ();
-
-	g_object_set (pl, "recurse", !option_no_recurse,
-			  "debug", option_debug,
-			  "force", option_force,
-			  "disable-unsafe", option_disable_unsafe,
-			  NULL);
-	g_signal_connect (G_OBJECT (pl), "entry-parsed",
-			  G_CALLBACK (entry_parsed_id_cb), &ret);
-
-	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
-	g_test_message ("Got retval %d for uri '%s'", retval, uri);
-	g_object_unref (pl);
-
-	return ret;
-}
-
-static void
-entry_parsed_download_cb (TotemPlParser *parser,
-		 const char *uri,
-		 GHashTable *metadata,
-		 char **ret)
-{
-	if (*ret == NULL)
-		*ret = g_strdup (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI));
-}
-
-static char *
-parser_test_get_download_uri (const char *uri)
-{
-	TotemPlParserResult retval;
-	char *ret = NULL;
-	TotemPlParser *pl = totem_pl_parser_new ();
-
-	g_object_set (pl, "recurse", !option_no_recurse,
-			  "debug", option_debug,
-			  "force", option_force,
-			  "disable-unsafe", option_disable_unsafe,
-			  NULL);
-	g_signal_connect (G_OBJECT (pl), "entry-parsed",
-			  G_CALLBACK (entry_parsed_download_cb), &ret);
-
-	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
-	g_test_message ("Got retval %d for uri '%s'", retval, uri);
-	g_object_unref (pl);
-
-	return ret;
+	g_assert_cmpstr (parser_test_get_playlist_uri ("itms://itunes.apple.com/gb/podcast/best-of-chris-moyles-enhanced/id142102961?ign-mpt=uo%3D4"), ==, "http://downloads.bbc.co.uk/podcasts/radio1/moylesen/rss.xml");
+	g_assert_cmpstr (parser_test_get_playlist_uri ("http://itunes.apple.com/gb/podcast/radio-1-mini-mix/id268491175?uo=4"), ==, "http://downloads.bbc.co.uk/podcasts/radio1/r1mix/rss.xml");
 }
 
 static void
@@ -509,13 +421,25 @@ test_lastfm_parsing (void)
 	g_test_bug ("625823");
 
 	uri = get_relative_uri (TEST_SRCDIR "old-lastfm-output.xspf");
-	g_assert_cmpstr (parser_test_get_download_uri (uri), ==, "http://freedownloads.last.fm/download/188024406/Kondratiev%2BWinter.mp3");
-	g_assert_cmpstr (parser_test_get_id (uri), ==, "d092a");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI), ==, "http://freedownloads.last.fm/download/188024406/Kondratiev%2BWinter.mp3");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_ID), ==, "d092a");
 	g_free (uri);
 
 	uri = get_relative_uri (TEST_SRCDIR "new-lastfm-output.xspf");
-	g_assert_cmpstr (parser_test_get_download_uri (uri), ==, "http://freedownloads.last.fm/download/402599273/Yellow.mp3");
-	g_assert_cmpstr (parser_test_get_id (uri), ==, "20a82");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI), ==, "http://freedownloads.last.fm/download/402599273/Yellow.mp3");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_ID), ==, "20a82");
+	g_free (uri);
+}
+
+static void
+test_m3u_separator (void)
+{
+	char *uri;
+
+	g_test_bug ("609091");
+
+	uri = get_relative_uri (TEST_SRCDIR "separator.m3u");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_TITLE), ==, "Music Tech Sessions (Friday 22 January 2010 20:00 - 00:00)");
 	g_free (uri);
 }
 
@@ -524,7 +448,52 @@ test_parsing_xspf_genre (void)
 {
 	char *uri;
 	uri = get_relative_uri (TEST_SRCDIR "playlist.xspf");
-	g_assert_cmpstr (parser_test_get_parse_genre (uri), ==, "Test Genre");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_GENRE), ==, "Test Genre");
+	g_free (uri);
+}
+
+static void
+test_smi_starttime (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "big5.smi");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_STARTTIME), ==, "00:04:00");
+	g_free (uri);
+}
+
+static void
+test_m3u_leading_tabs (void)
+{
+	char *uri;
+	/* From http://media.artistserver.com/tracks/23985/21898/1/1/1/O_G_Money_-_Girl_Gotta_girlfriend_Feat._O_G_Money_Snoop_Dogg.m3u */
+	uri = get_relative_uri (TEST_SRCDIR "O_G_Money_Snoop_Dogg.m3u");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_TITLE), ==, "O G Money - Girl Gotta girlfriend Feat. O G Money, Snoop Dogg");
+	g_free (uri);
+}
+
+static void
+test_directory_recurse (void)
+{
+	char *uri, *path;
+
+	uri = get_relative_uri (TEST_SRCDIR "foo");
+	path = g_filename_from_uri (uri, NULL, NULL);
+	if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
+		/* The file inside the directory will be ignored */
+		g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_TITLE), ==, NULL);
+		/* But the parsing will succeed */
+		g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	}
+	g_free (path);
+	g_free (uri);
+}
+
+static void
+test_empty_asx (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "empty-asx.asx");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
 	g_free (uri);
 }
 
@@ -534,7 +503,7 @@ test_parsing_rtsp_text_multi (void)
 	char *uri;
 	g_test_bug ("602127");
 	uri = get_relative_uri (TEST_SRCDIR "602127.qtl");
-	g_assert_cmpstr (parser_test_get_parse_result (uri), ==, "rtsp://host.org/video.mp4");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "rtsp://host.org/video.mp4");
 	g_free (uri);
 }
 
@@ -543,7 +512,7 @@ test_parsing_rtsp_text (void)
 {
 	char *uri;
 	uri = get_relative_uri (TEST_SRCDIR "single-line.qtl");
-	g_assert_cmpstr (parser_test_get_parse_result (uri), ==, "rtsp://host.org/video.mp4");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "rtsp://host.org/video.mp4");
 	g_free (uri);
 }
 
@@ -607,6 +576,28 @@ test_parsing_404_error (void)
 	g_assert (simple_parser_test ("http://live.hujjat.org:7860/main") == TOTEM_PL_PARSER_RESULT_UNHANDLED);
 }
 
+static void
+test_parsing_3gpp_not_ignored (void)
+{
+	char *uri;
+
+	uri = get_relative_uri (TEST_SRCDIR "3gpp-file.mp4");
+	g_test_bug ("594359@bugs.debian.org");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_free (uri);
+}
+
+static void
+test_parsing_mp4_is_flv (void)
+{
+	char *uri;
+
+	uri = get_relative_uri (TEST_SRCDIR "really-flv.mp4");
+	g_test_bug ("620039");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_free (uri);
+}
+
 static void
 test_parsing_xml_head_comments (void)
 {
@@ -840,6 +831,8 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/nonexistent_files", test_parsing_nonexistent_files);
 		g_test_add_func ("/parser/parsing/broken_asx", test_parsing_broken_asx);
 		g_test_add_func ("/parser/parsing/404_error", test_parsing_404_error);
+		g_test_add_func ("/parser/parsing/3gpp_not_ignored", test_parsing_3gpp_not_ignored);
+		g_test_add_func ("/parser/parsing/mp4_is_flv", test_parsing_mp4_is_flv);
 		g_test_add_func ("/parser/parsing/out_of_order_asx", test_parsing_out_of_order_asx);
 		g_test_add_func ("/parser/parsing/xml_head_comments", test_parsing_xml_head_comments);
 		g_test_add_func ("/parser/parsing/xml_comment_whitespace", test_parsing_xml_comment_whitespace);
@@ -854,6 +847,11 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/xspf_genre", test_parsing_xspf_genre);
 		g_test_add_func ("/parser/parsing/itms_link", test_itms_parsing);
 		g_test_add_func ("/parser/parsing/lastfm-attributes", test_lastfm_parsing);
+		g_test_add_func ("/parser/parsing/m3u_separator", test_m3u_separator);
+		g_test_add_func ("/parser/parsing/smi_starttime", test_smi_starttime);
+		g_test_add_func ("/parser/parsing/m3u_leading_tabs", test_m3u_leading_tabs);
+		g_test_add_func ("/parser/parsing/empty-asx.asx", test_empty_asx);
+		g_test_add_func ("/parser/parsing/dir_recurse", test_directory_recurse);
 
 		return g_test_run ();
 	}
