@@ -1,7 +1,8 @@
 
-/* Generated data (by glib-mkenums) */
+
 
 #include "totem-pl-parser.h"
+#include "totem-disc.h"
 #include "totem-pl-parser-builtins.h"
 
 /* enumerations from "totem-pl-parser.h" */
@@ -55,5 +56,25 @@ totem_pl_parser_error_get_type (void)
   return etype;
 }
 
-/* Generated data ends here */
+/* enumerations from "totem-disc.h" */
+GType
+totem_disc_media_type_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MEDIA_TYPE_ERROR, "MEDIA_TYPE_ERROR", "error" },
+      { MEDIA_TYPE_DATA, "MEDIA_TYPE_DATA", "data" },
+      { MEDIA_TYPE_CDDA, "MEDIA_TYPE_CDDA", "cdda" },
+      { MEDIA_TYPE_VCD, "MEDIA_TYPE_VCD", "vcd" },
+      { MEDIA_TYPE_DVD, "MEDIA_TYPE_DVD", "dvd" },
+      { MEDIA_TYPE_DVB, "MEDIA_TYPE_DVB", "dvb" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("TotemDiscMediaType", values);
+  }
+  return etype;
+}
+
+
 
