@@ -1888,7 +1888,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 					DEBUG(file, g_print ("URI '%s' dual type has type '%s' from data\n", uri, mimetype));
 				}
 				/* If it's _still_ a text/plain, we don't want it */
-				if (mimetype != NULL && strcmp (mimetype, "text/plain") == 0) {
+				if (mimetype != NULL && g_content_type_is_a (mimetype, "text/plain")) {
 					ret = TOTEM_PL_PARSER_RESULT_IGNORED;
 					g_free (mimetype);
 					mimetype = NULL;
@@ -1896,9 +1896,9 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 				}
 				/* Now look for the proper function to use */
 				func = totem_pl_parser_get_function_for_mimetype (mimetype);
-				if (func == NULL && mimetype != NULL) {
+				if ((func == NULL && mimetype != NULL) || (mimetype == NULL && dual_types[i].func == NULL)) {
 					DEBUG(file, g_print ("Ignoring URI '%s' because we couldn't find a playlist parser for '%s'\n", uri, mimetype));
-					ret = TOTEM_PL_PARSER_RESULT_IGNORED;
+					ret = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 					g_free (mimetype);
 					mimetype = NULL;
 					break;
